/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
 
package org.ow2.easywsdl.extensions.multiple.test;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.junit.Assert;
import org.ow2.easywsdl.extensions.multiple.MultipleExtFactory;
import org.ow2.easywsdl.extensions.multiple.api.Description;
import org.ow2.easywsdl.extensions.multiple.api.MultipleExtReader;
import org.ow2.easywsdl.extensions.multiple.api.MultipleExtWriter;
import org.ow2.easywsdl.extensions.sawsdl.SAWSDLFactory;
import org.ow2.easywsdl.extensions.wsdl4bpel.WSDL4BPELFactory;
import org.ow2.easywsdl.wsdl.ExtensionFactory;
import org.ow2.easywsdl.wsdl.WSDLFactory;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.WSDLReader;
import org.w3c.dom.Document;

/**
 * @author Nicolas Boissel-Dallier - EBM WebSourcing
 */
public class MultipleExtensionsTest extends TestCase{
    
    final private static URL WSDL_00_ALL_URL = Thread.currentThread().getContextClassLoader().getResource("wsdl/00-all.wsdl");
    
	public void testCreateDescFromMultipleExtReader() throws URISyntaxException, WSDLException, IOException {
		MultipleExtReader multExtReader = MultipleExtFactory.newInstance().newMultipleExtReader();
		List factories = this.getExtFactoriesList();
		Description desc = multExtReader.read(factories, WSDL_00_ALL_URL);
		Assert.assertNotNull(desc);
	}
	
	public void testCreateDescFromClassicReader() throws URISyntaxException, WSDLException, IOException {
		
		WSDLReader wsdlReader = WSDLFactory.newInstance().newWSDLReader();
		org.ow2.easywsdl.wsdl.api.Description wsdlDesc = wsdlReader.read(WSDL_00_ALL_URL);
		Description desc = null;
		
		List factories = this.getExtFactoriesList();
		desc = MultipleExtFactory.newInstance().addMultipleExtElmt2Description(factories, wsdlDesc);
		
		Assert.assertNotNull(desc);
	}
	
	public void testFindExpectedExtensions() throws WSDLException, URISyntaxException, IOException {
		// Create Multiple Description with all extensions
		MultipleExtReader multExtReader = MultipleExtFactory.newInstance().newMultipleExtReader();
		List factories = this.getExtFactoriesList();
		Description desc = multExtReader.read(factories, WSDL_00_ALL_URL);
		
		// Check SAWSDL Extension
		org.ow2.easywsdl.extensions.sawsdl.api.Description sawsdlDesc = desc.find(SAWSDLFactory.newInstance());
		Assert.assertEquals("[http://desc.example/, http://2.example]", sawsdlDesc.getModelReference().toString());
		
		// Check WSDL4BPEL Extension
		org.ow2.easywsdl.extensions.wsdl4bpel.api.Description wsdl4bpelDesc = desc.find(WSDL4BPELFactory.newInstance());
		Assert.assertEquals(new QName("http://greath.example.com/2004/wsdl/resSvc", "orderProcess"), wsdl4bpelDesc.getPartnerLinkTypes().get(0).getQName());
		
		// Check link between extensions
		sawsdlDesc.setTargetNamespace("http://easywsdl.ow2.org/changeSpread");
		Assert.assertEquals("http://easywsdl.ow2.org/changeSpread", wsdl4bpelDesc.getTargetNamespace());
	}
	
	public void testFindUnexpectedExtension() throws WSDLException, URISyntaxException, IOException {
		// Create Multiple Description with all extensions
		MultipleExtReader multExtReader = MultipleExtFactory.newInstance().newMultipleExtReader();
		List factories = new ArrayList<ExtensionFactory>();
		factories.add(SAWSDLFactory.newInstance());
		
		Description desc = multExtReader.read(factories, WSDL_00_ALL_URL);
		
		// Check SAWSDL Extension (expected)
		org.ow2.easywsdl.extensions.sawsdl.api.Description sawsdlDesc = desc.find(SAWSDLFactory.newInstance());
		Assert.assertEquals("[http://desc.example/, http://2.example]", sawsdlDesc.getModelReference().toString());
		
		// Check WSDL4BPEL Extension (non-expected)
		org.ow2.easywsdl.extensions.wsdl4bpel.api.Description wsdl4bpelDesc = desc.find(WSDL4BPELFactory.newInstance());
		Assert.assertEquals(new QName("http://greath.example.com/2004/wsdl/resSvc", "orderProcess"), wsdl4bpelDesc.getPartnerLinkTypes().get(0).getQName());
	}
	
	public void testManageTagsFromMultipleExt() throws WSDLException, IOException, URISyntaxException{
		// Create Multiple Description with all extensions
		MultipleExtReader multExtReader = MultipleExtFactory.newInstance().newMultipleExtReader();
		Description desc = multExtReader.read(WSDL_00_ALL_URL);
		
		// Add partnerLink
		org.ow2.easywsdl.extensions.wsdl4bpel.api.Description wsdl4bpelDesc = desc.find(WSDL4BPELFactory.newInstance());
		wsdl4bpelDesc.getPartnerLinkTypes().get(0).setQName(new QName("testJUnit"));
		Assert.assertEquals(new QName("http://greath.example.com/2004/wsdl/resSvc", "testJUnit"), wsdl4bpelDesc.getPartnerLinkTypes().get(0).getQName());

		// Add modelReference
		org.ow2.easywsdl.extensions.sawsdl.api.Description sawsdlDesc = desc.find(SAWSDLFactory.newInstance());
		sawsdlDesc.addModelReference(new URI("http://3.example"));
		Assert.assertEquals("[http://desc.example/, http://2.example, http://3.example]", sawsdlDesc.getModelReference().toString());
	}
	
	public void testWriteMultipleExtDesc() throws WSDLException, IOException, URISyntaxException{
		// Create Multiple Description with all extensions
		MultipleExtReader multExtReader = MultipleExtFactory.newInstance().newMultipleExtReader();
		Description desc = multExtReader.read(WSDL_00_ALL_URL);
		
		// Edit some parts
		org.ow2.easywsdl.extensions.wsdl4bpel.api.Description wsdl4bpelDesc = desc.find(WSDL4BPELFactory.newInstance());
		wsdl4bpelDesc.getPartnerLinkTypes().get(0).setQName(new QName("testJUnit"));
		org.ow2.easywsdl.extensions.sawsdl.api.Description sawsdlDesc = desc.find(SAWSDLFactory.newInstance());
		sawsdlDesc.addModelReference(new URI("http://3.example"));
		
		MultipleExtWriter multExtWriter = MultipleExtFactory.newInstance().newMultipleExtWriter();
		Document doc = multExtWriter.getDocument(desc);
		Assert.assertNotNull(doc);
		// System.out.println(XMLPrettyPrinter.prettyPrint(doc));
		
	}
	
	private List getExtFactoriesList() throws WSDLException {
		List<ExtensionFactory> factories = new ArrayList<ExtensionFactory>();
		factories.add(SAWSDLFactory.newInstance());
		factories.add(WSDL4BPELFactory.newInstance());
		return factories;
	}
}
